@@ -1,6 +1,9 @@
 from .models import *
+from .base import *
 from .automl_alex import *
 from .data_prepare import *
 from .encoders import *
 from .cross_validation import *
+from .optimizer import *
+from .logger import *
 from .__version__ import __version__
\ No newline at end of file
