@@ -3,12 +3,16 @@ from tqdm import tqdm
 import pandas as pd
 import time
 import joblib
+import automl_alex
 from .models import *
 from .cross_validation import *
 from .data_prepare import *
 from .encoders import *
+from .optimizer import *
 from pathlib import Path
+from .logger import *
 
+TMP_FOLDER = '.automl-alex_tmp/'
 
 ##################################### BestSingleModel ################################################
 
@@ -44,25 +48,26 @@ class ModelsReview(object):
             self._metric = metric
         
         self._metric_round = metric_round
-    
 
+
+    @logger.catch
     def fit(self,
         X_train=None, 
         y_train=None, 
         X_test=None, 
         y_test=None,
         models_names=None,
-        verbose=1,
+        verbose=3,
         ):
         """
         Fit models (in list models_names) whis default params
         """
-
+        logger_print_lvl(verbose)
         result = pd.DataFrame(columns=['Model_Name', 'Score', 'Time_Fit_Sec'])
         score_ls = []
         time_ls = []
         if models_names is None:
-            self.models_names = all_models.keys()
+            self.models_names = automl_alex.models.all_models.keys()
         else:
             self.models_names = models_names
 
@@ -75,12 +80,12 @@ class ModelsReview(object):
         for model_name in tqdm(self.models_names, disable=disable_tqdm):
             # Model
             start_time = time.time()
-            model_tmp = all_models[model_name](
+            model_tmp = automl_alex.models.all_models[model_name](
                                             gpu=self._gpu, 
                                             random_state=self._random_state,
                                             type_of_estimator=self.type_of_estimator)
             # fit
-            model_tmp = model_tmp.fit(X_train, y_train)
+            model_tmp.fit(X_train, y_train)
             # Predict
             if (self._metric.__name__ in predict_proba_metrics) and (model_tmp.is_possible_predict_proba()):
                 y_pred = model_tmp.predict_proba(X_test)
@@ -106,6 +111,23 @@ class ModelsReviewRegressor(ModelsReview):
     type_of_estimator='regression'
 
 
+##################################### BestSingleModel ##################################
+
+class BestSingleModel(Optimizer):
+    '''
+    Trying to find which model work best on our data
+    '''
+    __name__ = 'BestSingleModel'
+
+
+class BestSingleModelClassifier(BestSingleModel):
+    type_of_estimator='classifier'
+
+
+class BestSingleModelRegressor(BestSingleModel):
+    type_of_estimator='regression'
+
+
 ##################################### Stacking #########################################
 
 # in progress...
@@ -125,7 +147,7 @@ class AutoML(object):
                 metric_round=4,
                 combined_score_opt=True,
                 gpu=False, 
-                random_state=42
+                random_state=42,
                 ):
         self._gpu = gpu
         self._random_state = random_state
@@ -147,34 +169,39 @@ class AutoML(object):
         self._metric_round = metric_round
 
 
+    @logger.catch
     def fit(self,
         X, 
         y,
-        timeout=300, # optimization time in seconds
+        timeout=500, # optimization time in seconds
         auto_parameters=True,
-        folds=5,
+        folds=7,
         score_folds=2,
         opt_lvl=2,
         early_stoping=100,
         feature_selection=True,
-        verbose=1,
+        verbose=3,
         ):
+        logger_print_lvl(verbose)
         X_source = X.copy()
         ####################################################
         # STEP 0
         start_step_0 = time.time()
-        if verbose > 0:
-            print('> Start Fit Base Model')
+        logger.info('> Start Fit Base Model')
+        if timeout < 400:
+            logger.warning("! Not enough time to find the optimal parameters. \n \
+                    Please, Increase the 'timeout' parameter for normal optimization. (min 500 sec)")
 
         self.cat_features=X.columns[(X.dtypes == 'object') | (X.dtypes == 'category')]
         X[self.cat_features] = X[self.cat_features].astype('str')
         X.fillna(0, inplace=True)
         X[self.cat_features] = X[self.cat_features].astype('category')
 
-        self.model_1 = CatBoost(
+        self.model_1 = automl_alex.CatBoost(
             type_of_estimator=self.type_of_estimator, 
             random_state=self._random_state,
             gpu=self._gpu,
+            verbose=verbose,
             )
         self.model_1 = self.model_1.fit(X, y, cat_features=self.cat_features.tolist())
         X = None
@@ -182,10 +209,8 @@ class AutoML(object):
         ####################################################
         # STEP 1
         start_step_1 = time.time()
-
-        if verbose > 0:
-            print('> DATA PREPROC')
-        self.de_1 = DataPrepare(
+        logger.info('> DATA PREPROC')
+        self.de_1 = automl_alex.DataPrepare(
             cat_encoder_names=['OneHotEncoder', 'CountEncoder'],
             #outliers_threshold=3,
             normalization=True,
@@ -196,19 +221,41 @@ class AutoML(object):
         if self.de_1.cat_features is not None:
             X = X.drop(self.de_1.cat_features, axis = 1)
 
-        if verbose > 0:
-            print('> Start Fit Models 1')
-        # Model 2
-        # self.model_2 = LinearModel(
-        #     type_of_estimator=self.type_of_estimator, 
-        #     random_state=self._random_state,
-        #     )
-        # self.model_2 = self.model_2.fit(X, y)
+        params = {
+                'metric' : self.metric,
+                'metric_round' :self._metric_round,
+                'auto_parameters':auto_parameters,
+                'folds':folds,
+                'score_folds':score_folds,
+                'opt_lvl':opt_lvl,
+                'early_stoping':early_stoping,
+                'type_of_estimator':self.type_of_estimator,
+                'random_state':self._random_state,
+                'gpu':self._gpu,
+                #'iteration_check': False,
+                }
 
+        # logger.info(50*'#')
+        # logger.info('> Start Fit Models 2')
+        # logger.info(50*'#')
+        # # Model 2
+        # self.model_2 = automl_alex.BestSingleModel(
+        #     models_names = ['LinearModel',],
+        #     feature_selection=False,
+        #     **params,
+        #     )
+
+        # history = self.model_2.opt(X,y, timeout=100, verbose=verbose)
+        # self.model_2.save(name='model_2', folder=TMP_FOLDER,)
+
+        logger.info(50*'#')
+        logger.info('> Start Fit Models 3')
+        logger.info(50*'#')
         # Model 3
-        self.model_3 = MLP(
+        self.model_3 = automl_alex.MLP(
             type_of_estimator=self.type_of_estimator, 
-            random_state=self._random_state
+            random_state=self._random_state,
+            verbose=verbose,
             )
         self.model_3 = self.model_3.fit(X, y)
 
@@ -220,8 +267,7 @@ class AutoML(object):
         # STEP 2
         start_step_2 = time.time()
 
-        if verbose > 0:
-            print('> DATA PREPROC')
+        logger.info('> DATA PREPROC')
 
         self.de_2 = DataPrepare(
             cat_encoder_names=['HelmertEncoder','CountEncoder','HashingEncoder'],
@@ -233,13 +279,14 @@ class AutoML(object):
         X = self.de_2.fit_transform(X_source)
         #X = X.drop(self.de_2.cat_features, axis = 1)
 
-        if verbose > 0:
-            print('> Start Fit Models 2')
+        logger.info(50*'#')
+        logger.info('> Start Fit Models 4')
 
-        self.model_4 = CatBoost(
+        self.model_4 = automl_alex.CatBoost(
             type_of_estimator=self.type_of_estimator, 
             random_state=self._random_state,
             gpu=self._gpu,
+            verbose=verbose,
             )
 
         self.model_4 = self.model_4.fit(X, y)
@@ -251,67 +298,30 @@ class AutoML(object):
         # Model 2 - 3
         start_step_3 = time.time()
 
-        if verbose > 0:
-            print(50*'#')
-            print('> Start Opt Model')
+        logger.info(50*'#')
+        logger.info('> Start Fit Models 5')
+        logger.info(50*'#')
 
         time_to_opt = (timeout - (time.time()-start_step_0)) - 60
         time.sleep(0.1)
 
-        self.model_5 = LightGBM(
-            type_of_estimator=self.type_of_estimator, 
-            random_state=self._random_state,
-            gpu=self._gpu,
+        self.model_5 = automl_alex.BestSingleModel(
+            models_names = ['LightGBM', 'ExtraTrees'],
+            feature_selection=feature_selection,
+            **params,
             )
 
-        params = {'X':X,
-                'y':y,
-                'metric' : self.metric,
-                'combined_score_opt' : self._combined_score_opt,
-                'metric_round' :self._metric_round,
-                'auto_parameters':auto_parameters,
-                'folds':folds,
-                'score_folds':score_folds,
-                'opt_lvl':opt_lvl,
-                'early_stoping':early_stoping,
-                'feature_selection':feature_selection,
-                'timeout':time_to_opt, # optimization time in seconds
-                'iteration_check': False,
-                'verbose':verbose}
-
-        history = self.model_5.opt(**params)
-            
-        self._select_features_model_5 = list(history['columns'].iloc[0])
-        self.best_params_model_5 = history['model_param'].iloc[0]
-        folds=self.model_5._folds
-
-        self.model_5 = LightGBM(
-            type_of_estimator=self.type_of_estimator, 
-            random_state=self._random_state,
-            gpu=self._gpu,
-            )
-        self.model_5.model_param = self.best_params_model_5
-
-        self.cv_model_5 = CrossValidation(
-        estimator=self.model_5,
-        folds=folds,
-        n_repeats=1,
-        print_metric=False, 
-        random_state=self._random_state,
-        )
-        self.cv_model_5.fit(X[self._select_features_model_5], y,)
+        history = self.model_5.opt(X,y, timeout=time_to_opt, verbose=verbose)
+        self.model_5.save(name='model_5', folder=TMP_FOLDER,)
 
         total_step_4 = (time.time() - start_step_3)
 
         ####################################################
-
-        if verbose > 0:
-            print(50*'#')
-            print('> Finish!')
-
-        return(self)
+        logger.info(50*'#')
+        logger.info('> Finish!')
 
 
+    @logger.catch
     def predict(self, X=None, verbose=0):
         """
         Args:
@@ -329,6 +339,7 @@ class AutoML(object):
         X.fillna(0, inplace=True)
         X[self.cat_features] = X[self.cat_features].astype('category')
 
+        # MODEL 1
         self.predict_model_1 = self.model_1.predict_or_predict_proba(X)
 
         ####################################################
@@ -337,7 +348,11 @@ class AutoML(object):
         if self.de_1.cat_features is not None:
             X = X.drop(self.de_1.cat_features, axis = 1)
 
-        #self.predict_model_2 = self.model_2.predict_or_predict_proba(X)
+        # MODEL 2
+        # self.model_2 = self.model_2.load(name='model_2', folder=TMP_FOLDER,)
+        # self.predict_model_2 = self.model_2.predict(X)
+
+        # MODEL 3
         self.predict_model_3 = self.model_3.predict_or_predict_proba(X)
 
         ####################################################
@@ -345,11 +360,12 @@ class AutoML(object):
         X = self.de_2.transform(X_source, verbose=verbose)
         #X = X.drop(self.de_2.cat_features, axis = 1)
 
+        # MODEL 4
         self.predict_model_4 = self.model_4.predict_or_predict_proba(X)
         
-        ####################################################
-        # STEP 3
-        self.predict_model_5 = self.cv_model_5.predict_test(X[self._select_features_model_5])
+        # MODEL 5
+        self.model_5 = self.model_5.load(name='model_5', folder=TMP_FOLDER,)
+        self.predict_model_5 = self.model_5.predict(X)
         
         ####################################################
         # STEP 4
@@ -363,18 +379,21 @@ class AutoML(object):
         return (predicts)
 
 
+    @logger.catch
     def save(self, name='AutoML_dump', folder='./'):
         dir_tmp = folder+"AutoML_tmp/"
         Path(dir_tmp).mkdir(parents=True, exist_ok=True)
         self.de_1.save(name='DataPrepare_1_dump', folder=dir_tmp)
         self.de_2.save(name='DataPrepare_2_dump', folder=dir_tmp)
         joblib.dump(self, dir_tmp+'AutoML'+'.pkl')
-        self.cv_model_5.save(name='cv_model_5', folder=dir_tmp,)
+        #self.model_2.save(name='model_2', folder=dir_tmp,)
+        self.model_5.save(name='model_5', folder=dir_tmp,)
         shutil.make_archive(folder+name, 'zip', dir_tmp)
         shutil.rmtree(dir_tmp)
-        print('Save AutoML')
+        logger.info('Save AutoML')
 
 
+    @logger.catch
     def load(self, name='AutoML_dump', folder='./'):
         dir_tmp = folder+"AutoML_tmp/"
         Path(dir_tmp).mkdir(parents=True, exist_ok=True)
@@ -384,9 +403,10 @@ class AutoML(object):
         model.de_1 = model.de_1.load('DataPrepare_1_dump', folder=dir_tmp)
         model.de_2 = DataPrepare()
         model.de_2 = model.de_2.load('DataPrepare_2_dump', folder=dir_tmp)
-        model.cv_model_5 = model.cv_model_5.load(name='cv_model_5', folder=dir_tmp,)
+        #model.model_2 = model.model_2.load(name='model_2', folder=dir_tmp,)
+        model.model_5 = model.model_5.load(name='model_5', folder=dir_tmp,)
         shutil.rmtree(dir_tmp)
-        print('Load AutoML')
+        logger.info('Load AutoML')
         return(model)
 
 
